require "logger"

module Logger::Logging
  autoload :ColoredLogger, "logger/logging/colored_logger"
  autoload :ExtendedLogger, "logger/logging/extended_logger"
  autoload :Factory, "logger/logging/factory"

  def self.configure receiver, env: nil
    if env
      receiver.logger = Factory.(env)
    else
      receiver.logger = instance
    end
    nil
  end

  def self.instance
    @logger ||= Factory.()
  end

  def self.get receiver, logger = nil
    logger ||= instance

    if receiver.is_a? Module
      logger.progname = "(#{receiver.name})"
    else
      logger.progname = "(#{receiver.class.name})"
    end

    logger
  end

  def logger= logger
    if logger
      @logger = Logger::Logging.get self, logger
    end
  end

  def logger
    @logger or NullLogger
  end

  module NullLogger
    %i(data trace debug info warn error fatal unknown progname=).each do |log_level|
      define_singleton_method log_level do |*| end
    end
  end
end
