module Logger::Logging
  class ColoredLogger < ExtendedLogger
    ANSI_COLORS = %i(black red green yellow blue magenta cyan white)

    DEFAULT_PALETTE = {
      "ANY"   => -> msg { bg(:blue, :normal, fg(:white, :bright, msg)) },
      "FATAL" => -> msg { bg(:red, :bright, fg(:white, :bright, msg)) },
      "ERROR" => -> msg { fg(:red, :bright, msg) },
      "WARN"  => -> msg { fg(:yellow, :normal, msg) },
      "INFO"  => -> msg { fg(:green, :normal, msg) },
      "DEBUG" => -> msg { fg(:blue, :normal, msg) },
      "TRACE" => -> msg { fg(:magenta, :bright, msg) },
      "DATA"  => -> msg { fg(:cyan, :normal, msg) },
    }

    attr_writer :palette

    def bg *args
      col :bg, *args
    end

    def col fgbg, color_code, intensity_code, str
      color_num = ANSI_COLORS.index color_code
      intensity_num = { :normal => 0, :bright => 1 }.fetch intensity_code
      fgbg_num = { :fg => 3, :bg => 4 }.fetch fgbg
      "\e[#{intensity_num};#{fgbg_num}#{color_num}m#{str}\e[0m"
    end

    def fg *args
      col :fg, *args
    end

    def format_message severity, *args, progname, message
      formatter = palette.fetch severity
      colored_message = instance_exec message, &formatter
      colored_progname = instance_exec progname, &formatter
      super severity, *args, colored_progname, colored_message
    end

    def palette
      @palette or DEFAULT_PALETTE
    end
  end
end
