module Logger::Logging
  class Factory
    attr_reader :env

    def initialize env
      @env = env
    end

    def self.call env = nil
      env ||= ENV
      instance = new env
      instance.()
    end

    def call
      logger = logger_class.new device
      logger.level = level
      logger
    end

    def device
      @device ||=
        begin
          path = env.fetch "LOG_DEVICE" do "/dev/stdout" end
          if path.is_a? String then File.open path, "a" else path end
        end
    end

    def level
      @level ||=
        begin
          level_name = env.fetch "LOG_LEVEL" do "WARN" end
          level_name = level_name.upcase

          if Logger.const_defined? level_name
            Logger.const_get level_name
          else
            {
              "DATA" => -2,
              "TRACE" => -1,
            }.fetch level_name
          end
        end
    end

    def logger_class
      if coloring? then ColoredLogger else ExtendedLogger end
    end

    def coloring?
      return @coloring unless @coloring.nil?
      @coloring =
        begin
          use_colors = env.fetch "LOG_COLOR" do
            if device.tty? then "on" else "off" end
          end
          use_colors == "on"
        end
    end
  end
end
